<?php

namespace Drupal\screenshot\Form;

use Drupal\Component\Render\PlainTextOutput;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Utility\Token;
use Drupal\file\FileRepositoryInterface;
use Drupal\media\Entity\Media;
use Drupal\screenshot\Ajax\SendImageCommand;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * In view mode send image.
 */
class SendImageForm extends ConfigFormBase {

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface|null $typedConfigManager
   *   The typed config manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   The file url generator.
   * @param \Drupal\file\FileRepositoryInterface $fileRepository
   *   The file repository.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system.
   * @param \Drupal\Core\Utility\Token $token
   *   The token service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    protected $typedConfigManager,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly FileUrlGeneratorInterface $fileUrlGenerator,
    protected readonly FileRepositoryInterface $fileRepository,
    protected readonly FileSystemInterface $fileSystem,
    protected readonly Token $token,
  ) {
    parent::__construct($config_factory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('entity_type.manager'),
      $container->get('file_url_generator'),
      $container->get('file.repository'),
      $container->get('file_system'),
      $container->get('token'),
    );
  }

  /**
   * Save Image.
   */
  public function saveImage(string $selector, Request $request) {
    $encoded_image = $request->request->get('screenshot');
    $url = '';
    $fid = FALSE;
    $delta = $request->request->get("delta");
    $entity_type = $request->request->get("entity_type");
    $bundle = $request->request->get("bundle");
    $field_name = $request->request->get("field_name");
    $inputFid = "{$field_name}[$delta][fids]";
    $fieldConfig = $this->entityTypeManager->getStorage('field_config')
      ->load($entity_type . '.' . $bundle . '.' . $field_name);
    $file_directory = $request->request->get("file_directory");
    if (!empty($fieldConfig)) {
      $file_directory = $fieldConfig->getSetting('file_directory');
    }
    if (!empty($encoded_image)) {
      // Create image directory.
      $file_directory = trim($file_directory, '/');
      $destination = PlainTextOutput::renderFromHtml(
        $this->token->replace($file_directory, [])
      );
      $dir = $uri = $this->configFactory()->get('system.file')->get('default_scheme') . '://';
      if (!empty($destination)) {
        $uri .= $destination . '/';
        $path = $this->fileSystem->realpath($uri);
        if ($path) {
          $this->fileSystem->prepareDirectory($path, FileSystemInterface::CREATE_DIRECTORY);
        }
        else {
          $extract = explode('/', str_replace('\\', '/', $destination));
          foreach ($extract as $dirName) {
            $dir .= $dirName . '/';
            $path = $this->fileSystem->realpath($dir);
            $this->fileSystem->prepareDirectory($path, FileSystemInterface::CREATE_DIRECTORY);
          }
        }
      }
      // Convert image base64 to file.
      [$dataImage, $encoded_image] = explode(",", $encoded_image);
      [$tmp, $ext] = explode('/', str_replace([';', 'base64', ','], '', $dataImage));
      unset($tmp);
      $encoded_image = str_replace(' ', '+', $encoded_image);
      $decoded_image = base64_decode($encoded_image);
      $filename = date('ymd') . '_' . rand(1000, 9999) . '.' . $ext;
      // Saves a file to the specified destination and creates a database entry.
      $file = $this->fileRepository->writeData($decoded_image, $uri . $filename, FileSystemInterface::EXISTS_REPLACE);
      $fid = $file->id();
      $uri = $file->getFileUri();
      $url = $this->fileUrlGenerator->generate($uri)->toString();
      $uid = $this->currentUser()->id();
      $file->setOwnerId($uid);
      $file->setPermanent();
      $file->save();
      if ($entity_type == 'media') {
        $title = $this->t('Screenshot');
        $image_media = Media::create([
          'name' => $title,
          'bundle' => 'image',
          'uid' => $uid,
          'status' => TRUE,
          'field_media_image' => [
            'target_id' => $fid,
            'alt' => $title,
            'title' => $title,
          ],
        ]);
        $image_media->save();
        $inputFid = $field_name;
        $fid = $image_media->id();
      }
    }
    $response = new AjaxResponse();
    if (!empty($url)) {
      $response->addCommand(new SendImageCommand($selector, $url, $inputFid, $fid));
    }

    return $response;
  }

  /**
   * Form id.
   */
  public function getFormId() {
    return 'screenshot_sendImage_form';
  }

  /**
   * Get editable configuration name.
   */
  protected function getEditableConfigNames() {
    return [$this->getFormId() . '.setting'];
  }

}
